/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_flash.c,v 1.8 2002/11/02 22:31:51 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created: 06/25/2002
 *
 * Heavily reworked by: A. Sharp, Fri Jan 12 12:56:04 PST 2007
 *
 *-----------------------------------------------------------------
 */

#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include <fnmatch.h>

#include "../ssc-zebra/zebra.h"
#include "command.h"
#include "vtysh.h"

#include "nfxsh.h"

#include <fstab.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <math.h>
#include "../sm-chassis/cm-api.h"
#include "../sm-anpssc/anpssc-api.h"
#include "../sm-elog/elog-api.h"
#include "../sm-elog/elog-def.h"
#include "../ssc-nfxsh/cmd_audit.h"
#include "../ssc-genlib/fs.h"
#include OS_INCL
#include GENL_OS_INCL
#include "../ssc-timekeeper/timekeep-api.h"

#include "../ssc-zebra/vector.h"
#include "../ssc-zebra/fs-vector.h"
#include "../ssc-genlib/fs-api.h"
#include "../ssc-initial-config/initial-config.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

/*
 * directories to be created after initializing a CF card
 */
static char *init_dirs[] = {
	"dev",
	"tmp",
	"etc",
	"mnt",
	"var",
	"mnt1",
	"usr/local",
	"onstor/conf",
	NULL,
};

extern const char *primary_device;
extern const char *secondary_device;
extern const char *primary_disk;
extern const char *secondary_disk;
extern const char *fs_disk_parts;
extern char *secondary_mount;

/* old flash layout marker in genlib */
extern int layout_x;

/*
 * lists of files -- OS and hardware platform dependent
 */
extern char *sys_config_files;
extern char *all_files;
extern char *os_init_dirs[];

extern void write_label_preamble(char *dtab_name, int secs_cyl, int ncyl, uint64_t tot_blocks);
extern void write_root_entry(float root_sz, int secs_cyl);
extern void write_swap_entry(float swap_sz, float ppsize, int secs_cyl);
extern void write_whole_entry(uint64_t tot_blocks, int ncyl);
extern void write_var_entry(float var_sz, float ppsize, int secs_cyl);
extern void write_tmp_entry(float swap_sz, float ppsize, int secs_cyl);
extern void write_conf_entry(float conf_sz, float ppsize, int secs_cyl);
extern int disk_size(const char *device, uint64_t *disk_blocks, size_t *block_size, int *ncyls);
extern int mkfs(char part);
extern int mkswap(char part);
extern int do_disklabel(int ncyl, const char *device_name, char *label_name);


/*++

Routine Description:

    This routine creates the disk names

Arguments:

    ncyl        - The number of cylinders on the disk
    disk_blocks - The number of blocks on the disk
    bock_size - The number of bytes in a block
	dtab_name - The name of the file for writing out the disk label

Return Value:

    int - success or failure

--*/

FILE *dtf; /* file pointer used by os-specific output routines */

static int
disk_name(int ncyl, uint64_t disk_blocks, size_t block_size, char *dtab_name)
{
	int ret_val;
#ifdef COUGAR
# define CF_SIZE_MIN (980 * 2048)  /* 512MB flash with a few blocks missing */
#else
# define CF_SIZE_MIN (480 * 2048)  /* 512MB flash with a few blocks missing */
#endif

	/*****************************************************************
	 ***    WARNING: changes to this function must be mirrored in   **
	 ***             the flash_install.sh script!!!                 **
	 ***    You've been warned!!                                    **
	 *****************************************************************/

	sprintf(dtab_name, "cf%lld.%s", disk_blocks / 2048, DTEXT); /* MiB */
	printf("Secondary CF card is %lld MiB\n", disk_blocks / 2048);
	if (disk_blocks > CF_SIZE_MIN) {
		char fullname[128];

		sprintf(fullname, "/tmp/%s", dtab_name);
		/*
		 * open and truncate if it exists, or create
		 */
		dtf = fopen(fullname, "w+");
		if (dtf != NULL) {
			int secs;			/* sectors per cylinder */
			float psize;		/* partition size in cyls */
			float ppsize = 0.0;	/* sum of previous partition sizes */
			float root_sz;
			float tmp_sz;
			float swap_sz;
			float conf_sz;
			float var_sz;

			tmp_sz = 0.0;		/* shut the compiler up */
			secs = disk_blocks / ncyl;

			printf("Creating new disk label %s\n", dtab_name);
#define CYLS_PER_MB (1048576.0 / (((float)disk_blocks / (float)ncyl) * 512.0))

			write_label_preamble(dtab_name, secs, ncyl, disk_blocks);

			/*
			 * swap
			 * calculate size of swap.  should be about 6%, max 30MB
			 *
			 * round psize down to the nearest integer so our partitions are
			 * on cylinder boundaries.  Otherwise BSD wastes space.
			 */
			root_sz = ncyl;

			psize = floorf(.06 * ncyl);
			if (psize > (30.0 * CYLS_PER_MB)) {
				psize = floorf(30.0 * CYLS_PER_MB);
			}
			swap_sz = psize;
			root_sz = root_sz - psize;

#ifdef OpenBSD
			/*
			 * /tmp
			 * calculate size for tmp.  approx. 4%
			 */
			psize = floorf(.04 * ncyl);
			tmp_sz = psize;
			root_sz = root_sz - psize;
#endif

			/*
			 * /var - rw
			 * calculate size for var.  appr. 16%
			 */
			psize = floorf(VAR_SZ_PERCENT * ncyl);
			var_sz = psize;
			root_sz = root_sz - psize;

			/*
			 * /onstor/conf
			 * calculate the size of rw conf partition: 10%
			 */
			psize = floorf(ONSTOR_CONF_SZ_PERCENT * ncyl);
			conf_sz = psize;

			/*
			 * root - ro
			 * calculate root size (root, /usr and /onstor now folded together)
			 * root gets the remainder, about 64%
			 */
			root_sz = root_sz - psize;

			/*
			 * root - r0
			 * write root partition entry
			 */
			write_root_entry(root_sz, secs);
			ppsize = root_sz;

			/*
			 * swap
			 * write swap partition entry
			 */
			write_swap_entry(swap_sz, ppsize, secs);
			ppsize = ppsize + swap_sz;

			/*
			 * the /tmp and whole partitions only exist in BSD.
			 * rather than write empty write_{whole,tmp}_entry()
			 * functions for Linux, use ifdef's.
			 */
#ifdef OpenBSD
			/*
			 * whole
			 */
			fprintf(dtf, "  c: %8lld        0    unused        0     0        ",
				disk_blocks);
			fprintf(dtf, "# (Cyl.    0 - %4d)\n", ncyl - 1);
#endif

			/*
			 * /var - rw
			 * write var partition entry
			 */
			write_var_entry(var_sz, ppsize, secs);
			ppsize = ppsize + var_sz;

#ifdef OpenBSD
			/*
			 * /tmp - rw
			 * write tmp partition entry
			 */
			fprintf(dtf, "  e: %8d %8d    4.2BSD     1024  8192    16  ",
				(int)tmp_sz * secs, (int)ppsize * secs);
			fprintf(dtf, "# (Cyl. %4d - %4d)\n", (int)ppsize,
				(int)ppsize + (int)tmp_sz - 1);
			ppsize = ppsize + tmp_sz;
#endif

			/*
			 * /onstor/conf - rw
			 * write onstor conf partition entry
			 */
			write_conf_entry(conf_sz, ppsize, secs);
			ppsize = ppsize + conf_sz;

			/*
			 * finished creating label file, now move it over
			 */
			fclose(dtf);
			ret_val = CMD_SUCCESS;

		} else {
			fprintf(stderr, "Error opening file %s\n", fullname);
			ret_val = CMD_ERR_FAILED;
		}

		restore_fs_permissions("/");

		return ret_val;
	}

	printf("Disk size %lld bytes is not supported\n",
		disk_blocks * (uint64_t)block_size);

    return CMD_ERR_FAILED;
}


/*-----------------------------------------------------------------
 * Name :   create_filesystems()
 *
 * Description: Create filesystems for the devices
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created:    06/25/02
 *
 *----------------------------------------------------------------- */
static int
create_filesystems(void)
{
    int error = 0;
	char *p;

    for (p = (char *)fs_disk_parts; error == 0 && *p; p++) {
		error = mkfs(*p);
		if (error) {
			break;
		}
	}
    return error;
}


/*-----------------------------------------------------------------
 * Name :   do_copy_files_with_cp()
 *
 * Description: Copy the files from fromdir to todir with cp.
 *
 *-----------------------------------------------------------------
 */
static int
do_copy_files_with_cp(char *fromdir, char *todir, char *files)
{
    char *filelist;
    char *fname;
    int bufsize;

    bufsize = strlen(files);
    filelist = malloc(bufsize+1);
    if (filelist == NULL) {
        printf("%s: failed to allocate %d memory\n", __func__, bufsize);
        return CMD_ERR_FAILED;
    }
    strcpy(filelist, files);
    while ((fname = strsep(&filelist," \t")) != NULL) {
       printf("%s\n", fname);
       if (*fname == '/') {
           do_system("cp -R -L -f -p %s%s %s%s", fromdir, fname, todir, fname);
       } else {
           do_system("cp -R -L -f -p %s/%s %s/%s", fromdir, fname, todir,
				fname);
       }
    }
    free(filelist);
    return CMD_SUCCESS;
}


/*-----------------------------------------------------------------
 * Name :   tweak_filesystem()
 *
 * Description: Create some dirs and symlinks and other files
 *
 * Created by:  Andrew Sharp
 *
 * Date Created:    Thu Jan  4 18:31:50 PST 2007
 *
 *-----------------------------------------------------------------
 */
static int
tweak_filesystem(const char *disk_dev)
{
	char rdev[64];
	int err;
	char **flist;

	sprintf(rdev, "%s" ROOTFS_PART_LETTER, disk_dev);
    /*
	 * check if mount point directory exists. Else create it.
     */
	ensure_dir_exists(secondary_mount, 0755);
#ifdef DEBUG
	printf("mounting %s on %s:\n", rdev, secondary_mount);
#endif
	err = do_system("mount %s %s", rdev, secondary_mount);
	if (err) {
		fprintf(stderr, "%s: mount of %s on %s failed: %s(%d)\n", __func__,
			rdev, secondary_mount, strerror(err), err);
		return err;
	}
#ifdef DEBUG
	err = system("mount");
#endif
	/*
	 * make some essential directories that don't get made otherwise
	 */
	for (flist = init_dirs; ; flist = os_init_dirs) {
		char **p;

		p = &flist[0];
		do {
			if (!*p) {
				break;
			}
			do_system("mkdir -p %s/%s", secondary_mount, *p);
			p++;
		} while (1);
		if (flist == os_init_dirs) {
			break;
		}
	}

	if (err) {
		fprintf(stderr, "%s: mkdir failed: %s(%d)\n", __func__, strerror(err),
			err);
		do_system("umount %s", secondary_mount);
		return err;
	}

	/*
	 * OS specific filesystem tweeking
	 */
	OS_CF_TWEAKAGE;

	/*
	 * Special cookie to identify freshly created flash
	 */
	err = do_system("touch %s/onstor/.fresh-flash", secondary_mount);
	if (err) {
		fprintf(stderr, "%s: cookie creation failed: %s(%d)\n", __func__,
			strerror(err), err);
		do_system("umount %s", secondary_mount);
		return err;
	}

	/*
	 * we have a default fstab file called /etc/mdrive_fstab which is used
	 * by the rc file on certain occassions.  use that file to create the
	 * new fstab file.  that file should always have first CF slot as
	 * the device.
	 */
	do_system("sed 's/%s/%s/' /etc/mdrive_fstab > %s/etc/fstab", primary_device,
		secondary_device, secondary_mount);
	do_system("cp /etc/mdrive_fstab %s/etc/fstab", secondary_mount);
	do_system("umount %s", secondary_mount);

	return 0;
}


/*-----------------------------------------------------------------
 * Name :   do_copy_init()
 *
 * Description: Label and create the filesystems on the secondary disk
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created:    06/25/02
 *
 *-----------------------------------------------------------------
 */
int
do_copy_init(int argc, char **argv)
{
    uint64_t disk_blocks = 0;
    size_t block_size = 0;
	int ncyl = 0;
    int error = CMD_ERR_FAILED;

	become_root();

    error = disk_size(secondary_disk, &disk_blocks, &block_size, &ncyl);
    if (error == 0) {
        char name[32];

		/*
		 * make a disk label
		 */
        error = disk_name(ncyl, disk_blocks, block_size, name);
		/*
		 * write a disk label
		 */
		if (error == 0) {
            error = do_disklabel(ncyl, secondary_device, name);
		}
		if (error == 0) {
			error = create_filesystems();
		}
		if (error == 0) {
			error = mkswap(SWAP_PART_LETTER);
		}
		/*
		 * sundry tasks needed after erasing a CF card
		 */
		if (error == 0) {
			error = tweak_filesystem(secondary_disk);
		}
		if (error) {
			printf("Problem initializing CF card %s\n", secondary_disk);
        }
	} else {
		printf("Failed to get disk size of %s\n", secondary_disk);
	}

	unbecome_root();

    return error;
}


/*-----------------------------------------------------------------
 * Name :   copy_to_secondary()
 *
 * Description: Recursively copy the files specified by the argument
 * files to the secondary disk. If a file is a directory, all files
 * below it will be copied. If the flag tar_only is TRUE, only tar will
 * be used; otherwise cp will be used as an alternative if tar fails.
 * There are two options because tar internally fails if the /tmp partition
 * is corrupted, but cp would still work. When copying "all" files to the
 * secondary flash tar must be used, but for copying select system config
 * files, cp can be used. It is not necessary or even a good idea to copy
 * all files from the primary flash if it is corrupted. However, it may be
 * necessary to get the system config files from the primary flash as part
 * of the recovery process. The "system config copy" command uses cp as
 * an alternative if tar fails to do the copy. Note that cp cannot be used
 * to copy "all" files because it doesn't preserve hard links.
 *
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created:    06/25/02
 *
 *-----------------------------------------------------------------
 */
static int
copy_to_secondary(char *files, boolean tar_only)
{
    int error;
    vector v;
	char *on = NULL;

    become_root();

	ensure_fs_writable("/");

	v = make_secondary_fstabs(1);

	if (!v) {
		restore_fs_permissions("/");
		unbecome_root();
		return CMD_ERR_FAILED;
	}

	/*
	 * the user could have typed CTRL-Z and hosed a previously running
	 * copy-to-secondary, in which case the directories are still mounted.
	 * deal with it.
	 */
	u_mount_secondary(v, 0);

    if ((error = mount_secondary(v)) == 0) {
        printf("Mounted secondary disk\n");
        printf("Starting to copy the files to the secondary disk\n");
		/*
		 * if copying from a new, dynamic layout, to an old layout,
		 * -- unusual, but it could happen --
		 * don't copy the onstor directory just yet.
		 */
		if (layout_x == DYN_TO_OLD) {
			if (files == all_files) {

				on = strstr(files, "onstor");
				if (on) {
					/*
					 * erase the word 'onstor' from the file list
					 */
					memset(on, ' ', strlen("onstor"));
				}
			}
		} else if (layout_x & _TO_DYN) {
			/*
			 * if ANY files are copied to secondary, and it's not the
			 * old layout, then the cookie file is removed
			 */
			do_system("rm -f %s/onstor/.fresh-flash", secondary_mount);
		}

		/*
		 * do the actual copying
		 */
        error = do_copy_files("/", secondary_mount, files);
        if ((error != CMD_SUCCESS) && (tar_only == FALSE)) {
			/*
			 * dead path?
			 */
            printf("Error detected, trying again...\n");
            error = do_copy_files_with_cp("/", secondary_mount, files);
			printf("Done.\n");
        }
		/*
		 * if copying from a new, dynamic layout, to an old layout,
		 * -- unusual, but it could happen --
		 * NOW copy the onstor directory ... to usr/local/agile
		 */
		if ((layout_x == DYN_TO_OLD) && (files == all_files)) {
			char usr_local_agile[80];

			sprintf(usr_local_agile, "%s/usr/local/agile", secondary_mount);
			error = do_copy_files("/onstor", usr_local_agile, ".");

			/*
			 * restore the onstor name to the file list, just in case
			 * the user were to run something like copy all -i
			 * after running copy all
			 */
			memcpy(on, "onstor", strlen("onstor"));
		}
		u_mount_secondary(v, 1);
    }

	restore_fs_permissions("/");

    unbecome_root();

    free_fstab_vector(v);

    return error == 0 ? CMD_SUCCESS : CMD_ERR_FAILED;
}


/*-----------------------------------------------------------------
 * Name :   do_copy_all()
 *
 * Description: Copy the system from primary to secondary disk.
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created:    06/25/02
 *
 *-----------------------------------------------------------------
 */
int
do_copy_all(int argc, char **argv)
{
    return copy_to_secondary(all_files, TRUE);
}


/*-----------------------------------------------------------------
 * Name :   do_copy_config
 *
 * Description: Copy the system configuration files from primary to
 * secondary disk.
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created:    06/25/02
 *
 *-----------------------------------------------------------------
 */
int
do_copy_config(int argc, char **argv)
{
    return copy_to_secondary(sys_config_files, FALSE);
}


static int
y_or_n(void)
{
    int first, ch;
    first = ch = getchar();
    while (ch != '\n' && ch != EOF)
        ch = getchar();
    return (first == 'y' || first == 'Y');
}


/*-----------------------------------------------------------------
 * Name :   do_log
 *
 * Description: log the message
 *
 * Created by:  Sripal Surendiran
 *
 * Date Created:    01/04/2006
 *
 *-----------------------------------------------------------------
 */
static void
do_log(int priority, char *fmt, ...)
{
    va_list va;

    va_start(va, fmt);
    vsyslog(priority, fmt, va);
    va_end(va);
}


static void
my_log(char *string)
{
	do_log(LOG_INFO, string);
}


/*-----------------------------------------------------------------
 * Name :   do_restore_config()
 *
 * Description: Restore the system configuration files from secondary
 * disk.
 *
 * Created by:  Maxim Kozlovsky
 *
 * Date Created:    06/26/02
 *
 *-----------------------------------------------------------------
 */
int
do_restore_config(int argc, char **argv)
{
    int error = 0;
	vector v;

    printf("The changes in the configuration since the last "
           "save will be lost\n"
           "Do you wish to proceed (y/n)? ");
    fflush(stdout);

    if (y_or_n()) {

		become_root();
        /*
		 * Send node event before we shutdown_all() because we
		 * need eventd to still be running.
         */

		v = make_secondary_fstabs(1);
		if (!v) {
			error = CMD_ERR_FAILED;
			goto err_done;
		}

        nfxsh_send_node_down_event("Node going down for reboot! ('system config restore' issued from nfxsh).", 0);

		/*
		 * Send node event before we shutdown_all() because we
		 * need eventd to still be running.
		 */
		error = noisy_make_allfs_writable(my_log);
		if (error == 0) {
			u_mount_secondary(v, 0);
			error = mount_secondary(v);
			if (error == 0) {
				printf("Mounted secondary disk\n");
				shutdown_all();
				printf("Starting to copy the files to the primary disk\n");
				error = do_copy_files(secondary_mount, "/", sys_config_files);
				u_mount_secondary(v, 1);
				if (error == 0) {
					reboot_all();
				} else {
					printf("error occured trying to restore config files!\n");
				}
			}
			make_allfs_readonly();
		}
err_done:
		unbecome_root();
		free_fstab_vector(v);  /* can be safely called with v==NULL */
    }

    return error;
}


/*-----------------------------------------------------------------
 * Name :  pattern_unlink
 *
 * Description: unlinks file matching the pattern
 *
 * Created by:  Sripal Surendiran
 *
 * Date Created:    01/04/2006
 *
 * Reworked code to fix bad memory leak caused by not freeing memory
 * allocated by scandir C library function.
 * A. Sharp, Fri Jan 12 12:40:35 PST 2007
 *
 *-----------------------------------------------------------------
 */
static int
pattern_unlink(char *pathname, char *pattern)
{
    int count,loop;
    struct dirent **files = NULL;
    char del_file[200];
    int ret_val;

    /*
	 * scan all the files in the given directory
     */
    count = scandir(pathname, &files, NULL, alphasort);

    /*
	 * if no files found, return without deleting
     */
    if (count == 0) {
		if (files) {
			free(files);
		}
        return 0;
    }

	if (count < 0) {
		if (files) {
			free(files);
		}
        return -1;
	}

	ret_val = 0;
    for (loop = 0; loop < count; loop++) {
		/*
		 * unlink files matching the pattern
		 */

		if (ret_val == 0) {
			if (!fnmatch(pattern, files[loop]->d_name, 0)) {
				sprintf(del_file, "%s%s", pathname, files[loop]->d_name);
				ret_val = unlink(del_file);
			}
        }
		free(files[loop]);
    }

	free(files);

    return ret_val;
}


/*-----------------------------------------------------------------
 * Name :   do_reset_config()
 *
 * Description: Reset the system configuration to factory defaults
 *
 *
 *-----------------------------------------------------------------
 */
int do_reset_config(int argc, char **argv)
{
    FILE *fp;
    char log_buf[200];
    int ret_val;
    int err_val;
    int retain = FALSE;

    if ((argc == 1) && (!strcmp(argv[0], "-r"))) {
        retain = TRUE;
        printf("All system configuration except initial configuration "
               "parameters will be lost.\n");
    } else {
        printf("All system configuration will be lost.\n");
    }
    printf("Do you wish to proceed (y/n)? ");
    fflush(stdout);
    if (y_or_n()) {
        /*
		 * Send node event before we shutdown_all() because we need eventd to
		 * still be running.
         */
        nfxsh_send_node_down_event("Node going down for reboot! ('system "
                                   "config reset' issued from nfxsh).", 0);

        become_root();
        shutdown_all();

        if (retain != TRUE) {
            /* delete the semaphore (initial config status file)
             */
            ret_val = unlink("/onstor/conf/nasgwayinit.status");

            if (ret_val < 0) {
                err_val = errno;
                if (!(ret_val == ENOENT)) {
                    sprintf(log_buf, "removing nasgwayinit.status failed : %s",
						strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }
            }

        }

        /* remove audit entry in the crontab
         */
        if (audit_deleteCrontabEntry(NULL) == FALSE) {
            err_val = errno;
            sprintf(log_buf,
                    "removing audit entry in crontab "
                    "failed : %s",
                    strerror(errno));
            do_log(LOG_INFO, log_buf);
        }

        become_root();

        /*
		 * remove cluster database to start from scratch after reboot
		 */
        ret_val = unlink("/onstor/conf/cluster.db.DB0");
        if (ret_val == 0 || (ret_val == -1 && errno == ENOENT)) {
			/*
			 * as good as success
			 */

            ret_val = unlink("/onstor/conf/cluster.db.DBSYS1");
            if (ret_val < 0) {
                err_val = errno;
                if (!(ret_val == ENOENT)) {
                    sprintf(log_buf, "removing cluster.db.DBSYS1 failed : %s",
						strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }
            }

            /*
			 * Remove any clusDb upgrade files.  Note the files may also exist
             * with a .tmp extension, too.  (See also /onstor below.)
             */
            ret_val = pattern_unlink("/var/onstor/",
                                     "cluster.db.DB0.upgrade_backup*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf,
                        "removing all /var/onstor/.cluster.db.DB0."
                        "upgrade_backup* files failed : %s",
                        strerror(errno));
                do_log(LOG_INFO, log_buf);
            }

            ret_val = pattern_unlink("/onstor/conf/",
                                     "cluster.db.DB0.upgrade_backup*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf, "removing all cluster.db.DB0."
                        "upgrade_backup* files failed : %s",
                        strerror(errno));
                do_log(LOG_INFO, log_buf);
            }

            ret_val = pattern_unlink("/onstor/conf/", "smbd*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf, "removing all smbd* files failed : %s",
					strerror(errno));
                do_log(LOG_INFO, log_buf);
            }

            /*
			 * Remove all Kerberos configuration files
             */
            ret_val = pattern_unlink("/onstor/conf/", "krb5*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf, "removing all krb5* files failed : %s",
					strerror(errno));
                do_log(LOG_INFO, log_buf);
            }

            ret_val = pattern_unlink("/onstor/conf/", "cifs.dbase*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf, "removing all cifs.dbase* files failed : %s",
					strerror(errno));
                do_log(LOG_INFO, log_buf);
            }

            ret_val = pattern_unlink("/var/onstor/samba/", "secrets.*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf, "removing all krb5* files failed : %s",
					strerror(errno));
                do_log(LOG_INFO, log_buf);
            }

            ret_val = unlink("/onstor/conf/cluster.conf");
            if (ret_val < 0) {
                err_val = errno;
                if (!(err_val == ENOENT)) {
                    sprintf(log_buf, "removing cluster.conf file failed : %s",
						strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }
            }

            ret_val = unlink("/onstor/conf/stand_cluster");
            if (ret_val < 0) {
                err_val = errno;
                if (!(err_val == ENOENT)) {
                    sprintf(log_buf, "removing stand_cluster file failed : %s ",
						strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }
            }


            /*
			 * remove local elog config to start from scratch after reboot
			 */
            ret_val = ensure_fs_writable("/");
            if (ret_val != 0) {
				sprintf(log_buf, "remounting / read/write failed : %s",
					strerror(errno));
				do_log(LOG_INFO, log_buf);
            }

            ret_val = unlink("/onstor/etc/elog.conf");
            if (ret_val < 0) {
                err_val = errno;
                if (err_val != ENOENT) {
                    sprintf(log_buf, "removing etc/elog.conf file failed : %s",
						strerror(err_val));
                    do_log(LOG_INFO, log_buf);
                }
            }

            /*
			 * remove snmp config file to start from scratch after reboot
			 */
            ret_val = unlink("/onstor/etc/snmpd.conf");
            if (ret_val < 0) {
                err_val = errno;
                if (err_val != ENOENT) {
                    sprintf(log_buf, "removing snmpd.conf file failed : %s",
							strerror(err_val));
                    do_log(LOG_INFO, log_buf);
                }
            }

            /*
			 * Remove any clusDb upgrade files.  Note the files may also exist
             * with a .tmp extension, too.  (See also /var/onstor and
             * /onstor/conf above.)
             */
            ret_val = pattern_unlink("/onstor/",
                                     "cluster.db.DB0.upgrade_backup*");
            if (ret_val < 0) {
                err_val = errno;
                sprintf(log_buf, "removing all cluster.db.DB0."
                        "upgrade_backup* files failed : %s",
                        strerror(err_val));
                do_log(LOG_INFO, log_buf);
            }

			/*
			 * reset root's crontab
			 * preserve the first four lines as long as they are
			 * comments or lines starting with a capital letter,
			 * after that, only lines that start with a capital letter.
			 */
			ret_val = do_system("(crontab -l | head -n 4 | egrep '^[A-Z#]';"
					"crontab -l | sed '1,4d' | egrep '^[A-Z]' ) | "
					"crontab -");

			ret_val = copyFile(NTPCONFTEMPLATE, NTPCONF);
			if (ret_val < 0) {
				sprintf(log_buf,
						"restoring default %s  file failed : %s",
						 NTPCONF, strerror(errno));
				do_log(LOG_INFO, log_buf);
			}

            if (retain != TRUE) {

                /* remove default route */
                ret_val = RESET_DEF_ROUTE(DEF_ROUTE_FILE);
                if (ret_val < 0) {
                    err_val = errno;
                    if (!(err_val == ENOENT)) {
                        sprintf(log_buf, "removing %s file failed : %s",
							DEF_ROUTE_FILE, strerror(errno));
                        do_log(LOG_INFO, log_buf);
                    }
                }

                ret_val = unlink("/etc/resolv.conf");
                if (ret_val < 0) {
                    err_val = errno;
                    if (err_val != ENOENT) {
                        sprintf(log_buf,
                                "removing /etc/resolv.conf file failed : %s",
                            strerror(errno));
                        do_log(LOG_INFO, log_buf);
                    }
                }

                ret_val = unlink("/etc/hosts.user");
                if (ret_val < 0) {
                    err_val = errno;
                    if (err_val != ENOENT) {
                        sprintf(log_buf, "removing /etc/hosts.user failed : %s",
							 strerror(errno));
                        do_log(LOG_INFO, log_buf);
                    }
                }

                /*
				 * deleting sc0 and sc1 interface
                 */
				UNHINGE_IFACES

                /*
				 * blanking gateway name
				 */
                fp = fopen(HOSTNAME_FILE, "w");

                if (fp != NULL) {
                    fprintf(fp, "localhost\n");
                    fclose(fp);
                } else {
                    sprintf(log_buf, "blanking %s failed : %s", HOSTNAME_FILE,
                        strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }

                ret_val = copyFile("/etc/hosts.org", "/etc/hosts");
                if (ret_val < 0) {
                    sprintf(log_buf,
                        "restoring default /etc/hosts file failed : %s",
                        strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }

                /*
				 * reset the ntp config file
				 */
                ret_val = unlink("/etc/ntp.conf");
                if (ret_val < 0) {
                    err_val = errno;
                    if (err_val != ENOENT) {
                        sprintf(log_buf,
                            "removing /etc/ntp.conf file failed : %s",
                            strerror(errno));
                        do_log(LOG_INFO, log_buf);
                    }
                }

                /*
				 * updating /etc/ntp/step-tickers file
				 */
                fp = fopen(NTPTICKERS, "w");

                if (fp != NULL) {
                    fprintf(fp, "%s\n", NTP_STEP_STICKERS_LOCALHOST);
                    fclose(fp);
                } else {
                    sprintf(log_buf, "updating %s failed : %s",
                            NTPTICKERS, strerror(errno));
                    do_log(LOG_INFO, log_buf);
                }
            }

			RESET_PASSWORD_FILE

            /* restore / to read only mode */
            restore_fs_permissions("/");

			/*
			 * clean log files
			 */
			pattern_unlink("/var/log/onstor", "messages*");
			do_system("touch /var/log/onstor/messages");
			pattern_unlink("/var/log", "auth.log.*");
			pattern_unlink("/var/log", "daemon.log.*");
			pattern_unlink("/var/log", "dmesg.*");
			pattern_unlink("/var/log", "debug.*");
			pattern_unlink("/var/log", "kern.log.*");
			pattern_unlink("/var/log", "mail.*");
			pattern_unlink("/var/log", "messages.*");
			pattern_unlink("/var/log", "syslog.*");

			/*
			 * OS specific detretius
			 */
			OS_VAR_CLEAN;
        } else {
            snprintf(log_buf, 200,
                     "Warning: Unable to delete %s ... resuming" ,
                     "/onstor/conf/cluster.db.DB0");
            printf("%s\n", log_buf);
            do_log(LOG_INFO, log_buf);
        }
        reboot_all();
        unbecome_root();
    }

    return CMD_SUCCESS;
}

/*
 * vi:ts=4 sw=4
 */
